# -*- coding: utf-8 -*-
"""
Test parsing of strings that are phrases
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    # a special compare function for nlp returned data

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, dateOnly=False):
        target = result
        value = check

        if target is None and value is None:
            return True

        if (target is None and value is not None) or \
                (target is not None and value is None):
            return False

        if len(target) != len(value):
            return False

        for i in range(0, len(target)):
            target_date = target[i][0]
            value_date = value[i][0]

            if target_date.year != value_date.year or \
                    target_date.month != value_date.month or \
                    target_date.day != value_date.day or \
                    target_date.hour != value_date.hour or \
                    target_date.minute != value_date.minute:
                return False
            if target[i][1] != value[i][1]:
                return False
            if target[i][2] != value[i][2]:
                return False
            if target[i][3] != value[i][3]:
                return False
            if target[i][4] != value[i][4]:
                return False

        return True

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testLongPhrase(self):
        # note: these tests do not need to be as dynamic as the others because
        #       this is still based on the parse() function, so all tests of
        #       the actual processing of the datetime value returned are
        #       applicable to this. Here we are concerned with ensuring the
        #       correct portions of text and their positions are extracted and
        #       processed.
        start = datetime.datetime(2013, 8, 1, 21, 25, 0).timetuple()
        target = ((datetime.datetime(2013, 8, 5, 20, 0),
                   3, 17, 37, 'At 8PM on August 5th'),
                  (datetime.datetime(2013, 8, 9, 21, 0),
                   3, 72, 90, 'next Friday at 9PM'),
                  (datetime.datetime(2013, 8, 1, 21, 30, 0),
                   2, 120, 132, 'in 5 minutes'),
                  (datetime.datetime(2013, 8, 8, 9, 0),
                   1, 173, 182, 'next week'))

        # positive testing
        self.assertExpectedResult(self.cal.nlp(
            "I'm so excited!! At 8PM on August 5th i'm going to fly to "
            "Florida. Then next Friday at 9PM i'm going to Dog n Bone! "
            "And in 5 minutes I'm going to eat some food! Talk to you "
            "next week.", start), target)

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 17, 0, 0).timetuple()

    def testQuotes(self):
        # quotes should not interfere with datetime language recognition
        start = datetime.datetime(2013, 8, 1, 21, 25, 0).timetuple()
        target = self.cal.nlp(
            "I'm so excited!! At '8PM on August 5th' i'm going to fly to "
            "Florida. Then 'next Friday at 9PM' i'm going to Dog n Bone! "
            "And in '5 minutes' I'm going to eat some food! Talk to you "
            '"next week"', start)

        self.assertEqual(target[0][4], "At '8PM on August 5th")
        self.assertEqual(target[1][4], "next Friday at 9PM")
        self.assertEqual(target[2][4], "in '5 minutes")
        self.assertEqual(target[3][4], "next week")

    def testPrefixes(self):
        # nlp has special handling for on/in/at prefixes
        start = datetime.datetime(2013, 8, 1, 21, 25, 0).timetuple()

        target = self.cal.nlp("Buy a balloon on Monday", start)
        self.assertEqual(target[0][4], "on Monday")

        target = self.cal.nlp("Buy a balloon at noon", start)
        self.assertEqual(target[0][4], "at noon")

        target = self.cal.nlp("Buy a balloon in a month", start)
        self.assertEqual(target[0][4], "in a month")

        # Should no longer pull "on" off the end of balloon
        target = self.cal.nlp("Buy a balloon Monday", start)
        self.assertEqual(target[0][4], "Monday")

    def testFalsePositives(self):
        # negative testing - no matches should return None
        start = datetime.datetime(2013, 8, 1, 21, 25, 0).timetuple()
        self.assertExpectedResult(self.cal.nlp(
            "Next, I'm so excited!! So many things that are going to "
            "happen every week!!", start), None)
        self.assertExpectedResult(self.cal.nlp("$300", start), None)
        self.assertExpectedResult(self.cal.nlp("300ml", start), None)
        self.assertExpectedResult(self.cal.nlp("nice ass", start), None)



# -*- coding: utf-8 -*-
"""
Internal helper functions for unit tests of parsedatetime
"""
from __future__ import unicode_literals


def assertEqualWithComparator(comparator):
    """
    Fail a little less cryptically that unittest.assertTrue when comparing a
    result against a target value. Shows the result and the target in the
    failure message.
    """

    def decoratedComparator(self, result, check, errMsg=None, **kwargs):
        errMsg = errMsg or 'Result does not match target value'
        equal = comparator(self, result, check, **kwargs)
        failureMessage = ('%s\n\n\t'
                          'Result:\n\t%s\n\n\tExpected:\n\t%s')

        if not equal:
            self.fail(failureMessage % (errMsg, result, check))

    return decoratedComparator


def compareResultByTimeTuplesAndFlags(result, check, dateOnly=False):
    """
    Ensures that flags are an exact match and time tuples a close match when
    given data in the format ((timetuple), flag)
    """
    return (_compareTimeTuples(result[0], check[0], dateOnly) and
            _compareFlags(result[1], check[1]))


def compareResultByFlags(result, check, dateOnly=False):
    """
    Ensures that flags are an exact match when given data in the format
    ((timetuple), flag)
    """
    return _compareFlags(result[1], check[1])


def compareResultByTimeTupleRangesAndFlags(result, check, dateOnly=False):
    """
    Ensures that flags are an exact match and time tuples a close match when
    given data in the format ((timetuple), (timetuple), flag)
    """
    return (_compareTimeTuples(result[0], check[0], dateOnly) and
            _compareTimeTuples(result[1], check[1], dateOnly) and
            _compareFlags(result[2], check[2]))


def _compareTimeTuples(target, value, dateOnly=False):
    """
    Ignores minutes and seconds as running the test could cross a minute
    boundary. Technically the year, month, day, hour, minute, and second could
    all change if the test is run on New Year's Eve, but we won't worry about
    less than per-hour granularity.
    """
    t_yr, t_mth, t_dy, t_hr, t_min, _, _, _, _ = target
    v_yr, v_mth, v_dy, v_hr, v_min, _, _, _, _ = value

    if dateOnly:
        return ((t_yr == v_yr) and (t_mth == v_mth) and (t_dy == v_dy))
    else:
        return ((t_yr == v_yr) and (t_mth == v_mth) and (t_dy == v_dy) and
                (t_hr == v_hr) and (t_min == v_min))


def _compareFlags(result, check):
    return (result == check)



"""
Unit tests for parsedatetime

The tests can be run as a C{suite} by running::

    nosetests

Requires Python 3.0 or later
"""
import logging


__author__ = 'Mike Taylor (bear@code-bear.com)'
__copyright__ = 'Copyright (c) 2004 Mike Taylor'
__license__ = 'Apache v2.0'
__version__ = '1.0.0'
__contributors__ = ['Darshana Chhajed',
                    'Michael Lim (lim.ck.michael@gmail.com)',
                    'Bernd Zeimetz (bzed@debian.org)',
                    ]


log = logging.getLogger('parsedatetime')
echoHandler = logging.StreamHandler()
echoFormatter = logging.Formatter('%(levelname)-8s %(message)s')
log.addHandler(echoHandler)

# log.setLevel(logging.DEBUG)



# -*- coding: utf-8 -*-
"""
Test parsing of 'simple' offsets
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testHoursFromNow(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(hours=5)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('5 hours from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hour from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hr from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('in 5 hours', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('in 5 hour', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('5 hr', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('5h', start), (target, 2))

        self.assertExpectedResult(
            self.cal.parse('five hours from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five hour from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five hr from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('in five hours', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('in five hour', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five hours', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five hr', start), (target, 2))

        # Test "an"
        t = s + datetime.timedelta(hours=1)
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('an hour from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('in an hour', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('an hour', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('an hr', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('an h', start), (target, 2))

        # No match, should require a word boundary
        self.assertExpectedResult(self.cal.parse('anhour', start), (start, 0))
        self.assertExpectedResult(
            self.cal.parse('an hamburger', start), (start, 0))

    def testHoursBeforeNow(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(hours=-5)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('5 hours before now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hr before now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5h before now', start), (target, 2))

        self.assertExpectedResult(
            self.cal.parse('five hours before now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five hr before now', start), (target, 2))

        # Test "an"
        t = s + datetime.timedelta(hours=-1)
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('an hour before now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('an hr before now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('an h before now', start), (target, 2))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test parsing of simple date and times using the German locale
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.ptc = pdt.Constants('de_DE', usePyICU=False)
        self.cal = pdt.Calendar(self.ptc)

        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

        if self.ptc.localeID != 'de_DE':
            raise unittest.SkipTest(
                'Locale not set to de_DE - check if PyICU is installed')

    def testTimes(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            self.yr, self.mth, self.dy, 23, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('23:00:00', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('23:00', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('2300', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 11, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('11:00:00', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11:00', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1100', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 7, 30, 0).timetuple()

        self.assertExpectedResult(self.cal.parse('730', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('0730', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 17, 30, 0).timetuple()

        self.assertExpectedResult(self.cal.parse('1730', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('173000', start), (target, 2))

    def testDates(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            2006, 8, 25, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('25.08.2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('25.8.06', start), (target, 1))

        if self.mth > 8 or (self.mth == 8 and self.dy > 25):
            target = datetime.datetime(
                self.yr + 1, 8, 25, self.hr, self.mn, self.sec).timetuple()
        else:
            target = datetime.datetime(
                self.yr, 8, 25, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('25.8', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('25.08', start), (target, 1))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test parsing of complex date and times
"""
from __future__ import unicode_literals

import sys
import time
from datetime import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testDate3ConfusedHourAndYear(self):
        start = datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('Aug 05, 2014 4:15 AM'),
            (datetime(2014, 8, 5, 4, 15, 0).timetuple(), 3))
        self.assertExpectedResult(
            self.cal.parse('Aug 05, 2003 3:15 AM'),
            (datetime(2003, 8, 5, 3, 15, 0).timetuple(), 3))
        self.assertExpectedResult(
            self.cal.parse('Aug 05, 2003 03:15 AM'),
            (datetime(2003, 8, 5, 3, 15, 0).timetuple(), 3))
        self.assertExpectedResult(
            self.cal.parse('June 30th 12PM', start),
            (datetime(self.yr
                      if datetime.now() < datetime(self.yr, 6, 30, 12)
                      else self.yr + 1,
                      6, 30, 12, 0, 0).timetuple(), 3))
        self.assertExpectedResult(
            self.cal.parse('June 30th 12:00', start),
            (datetime(self.yr
                      if datetime.now() < datetime(self.yr, 6, 30, 12)
                      else self.yr + 1,
                      6, 30, 12, 0, 0).timetuple(), 3))
        self.assertExpectedResult(
            self.cal.parse('December 30th 23PM', start),
            (datetime(self.yr
                      if datetime.now() < datetime(self.yr, 12, 30, 23)
                      else self.yr + 1,
                      12, 30, 23, 0, 0).timetuple(), 3))
        self.assertExpectedResult(
            self.cal.parse('December 30th 23:02', start),
            (datetime(self.yr
                      if datetime.now() < datetime(self.yr, 12, 30, 23, 2)
                      else self.yr + 1,
                      12, 30, 23, 2, 0).timetuple(), 3))

    def testDates(self):
        start = datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime(2006, 8, 25, 17, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('08/25/2006 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5pm on 08.25.2006', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5pm August 25, 2006', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5pm August 25th, 2006', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5pm 25 August, 2006', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5pm 25th August, 2006', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('Aug 25, 2006 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('Aug 25th, 2006 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('25 Aug, 2006 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('25th Aug 2006, 5pm', start), (target, 3))

        if self.mth > 8 or (self.mth == 8 and self.dy > 5):
            target = datetime(self.yr + 1, 8, 5, 17, 0, 0).timetuple()
        else:
            target = datetime(self.yr, 8, 5, 17, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('8/5 at 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5pm 8.5', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('08/05 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('August 5 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5pm Aug 05', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('Aug 05 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('Aug 05th 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5 August 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5th August 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('5pm 05 Aug', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('05 Aug 5pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('05th Aug 5pm', start), (target, 3))

        self.assertExpectedResult(
            self.cal.parse('August 5th 5pm', start), (target, 3))

        if self.mth > 8 or (self.mth == 8 and self.dy > 5):
            target = datetime(self.yr + 1, 8, 5, 12, 0, 0).timetuple()
        else:
            target = datetime(self.yr, 8, 5, 12, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('August 5th 12:00', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('August 5th 12pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('August 5th 12:00pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('August 5th 12 pm', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('August 5th 12:00 pm', start), (target, 3))

        if self.mth > 8 or (self.mth == 8 and self.dy > 22):
            target = datetime(
                self.yr + 1, 8, 22, 3, 26, 0).timetuple()
        else:
            target = datetime(self.yr, 8, 22, 3, 26, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('August 22nd 3:26', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('August 22nd 3:26am', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('August 22nd 3:26 am', start), (target, 3))

    def testDatesWithDay(self):
        start = datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime(2016, 8, 23, 17, 0, 0).timetuple()
        self.assertExpectedResult(
            self.cal.parse('tuesday august 23nd 2016 at 5pm', start), (target, 3))

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test parsing of strings that are phrases with the
ptc.StartTimeFromSourceTime flag set to True
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        self.cal.ptc.StartTimeFromSourceTime = True
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testEndOfPhrases(self):
        s = datetime.datetime.now()

        # find out what month we are currently on
        # set the day to 1 and then go back a day
        # to get the end of the current month
        (yr, mth, dy, hr, mn, sec, _, _, _) = s.timetuple()

        s = datetime.datetime(yr, mth, dy, 13, 14, 15)

        mth += 1
        if mth > 12:
            mth = 1
            yr += 1
        t = datetime.datetime(
            yr, mth, 1, 13, 14, 15) + datetime.timedelta(days=-1)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(self.cal.parse('eom', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('meeting eom', start), (target, 1))

        s = datetime.datetime.now()

        (yr, mth, dy, hr, mn, sec, wd, yd, isdst) = s.timetuple()

        s = datetime.datetime(yr, mth, 1, 13, 14, 15)
        t = datetime.datetime(yr, 12, 31, 13, 14, 15)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(self.cal.parse('eoy', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('meeting eoy', start), (target, 1))



# -*- coding: utf-8 -*-
"""
Test parsing of simple date and times
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTupleRangesAndFlags(
            result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testTimes(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()

        targetStart = datetime.datetime(
            self.yr, self.mth, self.dy, 14, 0, 0).timetuple()
        targetEnd = datetime.datetime(
            self.yr, self.mth, self.dy, 17, 30, 0).timetuple()

        self.assertExpectedResult(self.cal.evalRanges(
            "2 pm - 5:30 pm", start), (targetStart, targetEnd, 2))
        self.assertExpectedResult(self.cal.evalRanges(
            "2pm - 5:30pm", start), (targetStart, targetEnd, 2))
        self.assertExpectedResult(self.cal.evalRanges(
            "2:00:00 pm - 5:30:00 pm", start), (targetStart, targetEnd, 2))
        self.assertExpectedResult(self.cal.evalRanges(
            "2 - 5:30pm", start), (targetStart, targetEnd, 2))
        self.assertExpectedResult(self.cal.evalRanges(
            "14:00 - 17:30", start), (targetStart, targetEnd, 2))

        targetStart = datetime.datetime(
            self.yr, self.mth, self.dy, 10, 0, 0).timetuple()
        targetEnd = datetime.datetime(
            self.yr, self.mth, self.dy, 13, 30, 0).timetuple()

        self.assertExpectedResult(self.cal.evalRanges(
            "10AM - 1:30PM", start), (targetStart, targetEnd, 2))
        self.assertExpectedResult(self.cal.evalRanges(
            "10:00:00 am - 1:30:00 pm", start), (targetStart, targetEnd, 2))
        self.assertExpectedResult(self.cal.evalRanges(
            "10:00 - 13:30", start), (targetStart, targetEnd, 2))

        targetStart = datetime.datetime(
            self.yr, self.mth, self.dy, 15, 30, 0).timetuple()
        targetEnd = datetime.datetime(
            self.yr, self.mth, self.dy, 17, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.evalRanges("today 3:30-5PM", start),
            (targetStart, targetEnd, 2))

    def testDates(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()

        targetStart = datetime.datetime(
            2006, 8, 29, self.hr, self.mn, self.sec).timetuple()
        targetEnd = datetime.datetime(
            2006, 9, 2, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.evalRanges("August 29, 2006 - September 2, 2006", start),
            (targetStart, targetEnd, 1))
        self.assertExpectedResult(
            self.cal.evalRanges("August 29 - September 2, 2006", start),
            (targetStart, targetEnd, 1))

        targetStart = datetime.datetime(
            2006, 8, 29, self.hr, self.mn, self.sec).timetuple()
        targetEnd = datetime.datetime(
            2006, 9, 2, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.evalRanges("08/29/06 - 09/02/06", start),
            (targetStart, targetEnd, 1))

    def _testSubRanges(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()

        targetStart = datetime.datetime(2006, 8, 1, 9, 0, 0).timetuple()
        targetEnd = datetime.datetime(2006, 8, 15, 9, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.evalRanges("August 1-15, 2006", start),
            (targetStart, targetEnd, 1))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test Calendar.Inc() routine
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags((result, 1),
                                                       (check, 1), **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testIncMonths(self):
        s = datetime.datetime(2006, 1, 1, 12, 0, 0)
        t = datetime.datetime(2006, 2, 1, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, month=1).timetuple(), t.timetuple())

        s = datetime.datetime(2006, 12, 1, 12, 0, 0)
        t = datetime.datetime(2007, 1, 1, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, month=1).timetuple(), t.timetuple())

        # leap year, Feb 1
        s = datetime.datetime(2008, 2, 1, 12, 0, 0)
        t = datetime.datetime(2008, 3, 1, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, month=1).timetuple(), t.timetuple())

        # leap year, Feb 29
        s = datetime.datetime(2008, 2, 29, 12, 0, 0)
        t = datetime.datetime(2008, 3, 29, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, month=1).timetuple(), t.timetuple())

        s = datetime.datetime(2006, 1, 1, 12, 0, 0)
        t = datetime.datetime(2005, 12, 1, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, month=-1).timetuple(), t.timetuple())

        # End of month Jan 31 to Feb - Febuary only has 28 days
        s = datetime.datetime(2006, 1, 31, 12, 0, 0)
        t = datetime.datetime(2006, 2, 28, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, month=1).timetuple(), t.timetuple())

        # walk thru months and make sure month increment doesn't set the day
        # to be past the last day of the new month
        # think Jan transition to Feb - 31 days to 28 days
        for m in range(1, 11):
            d = self.cal.ptc.daysInMonth(m, 2006)
            s = datetime.datetime(2006, m, d, 12, 0, 0)

            if d > self.cal.ptc.daysInMonth(m + 1, 2006):
                d = self.cal.ptc.daysInMonth(m + 1, 2006)

            t = datetime.datetime(2006, m + 1, d, 12, 0, 0)

            self.assertExpectedResult(
                self.cal.inc(s, month=1).timetuple(), t.timetuple())

    def testIncYears(self):
        s = datetime.datetime(2006, 1, 1, 12, 0, 0)
        t = datetime.datetime(2007, 1, 1, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, year=1).timetuple(), t.timetuple())

        s = datetime.datetime(2006, 1, 1, 12, 0, 0)
        t = datetime.datetime(2008, 1, 1, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, year=2).timetuple(), t.timetuple())

        s = datetime.datetime(2006, 12, 31, 12, 0, 0)
        t = datetime.datetime(2007, 12, 31, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, year=1).timetuple(), t.timetuple())

        s = datetime.datetime(2006, 12, 31, 12, 0, 0)
        t = datetime.datetime(2005, 12, 31, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, year=-1).timetuple(), t.timetuple())

        s = datetime.datetime(2008, 3, 1, 12, 0, 0)
        t = datetime.datetime(2009, 3, 1, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, year=1).timetuple(), t.timetuple())

        s = datetime.datetime(2008, 3, 1, 12, 0, 0)
        t = datetime.datetime(2007, 3, 1, 12, 0, 0)
        self.assertExpectedResult(
            self.cal.inc(s, year=-1).timetuple(), t.timetuple())


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test parsing of units
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    @utils.assertEqualWithComparator
    def assertExpectedErrorFlag(self, result, check, **kwargs):
        return utils.compareResultByFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testErrors(self):
        s = datetime.datetime.now()
        start = s.timetuple()

        # These tests all return current date/time as they are out of range
        self.assertExpectedResult(self.cal.parse('01/0', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('08/35', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('18/35', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('1799', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('781', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('2702', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('78', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('11', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('1', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('174565', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('177505', start), (start, 0))
        # ensure short month names do not cause false positives within a word -
        # jun (june)
        self.assertExpectedResult(
            self.cal.parse('injunction', start), (start, 0))
        # ensure short month names do not cause false positives at the start of
        # a word - jul (juuly)
        self.assertExpectedResult(self.cal.parse('julius', start), (start, 0))
        # ensure short month names do not cause false positives at the end of a
        # word - mar (march)
        self.assertExpectedResult(self.cal.parse('lamar', start), (start, 0))
        # ensure short weekday names do not cause false positives within a word
        # - mon (monday)
        self.assertExpectedResult(
            self.cal.parse('demonize', start), (start, 0))
        # ensure short weekday names do not cause false positives at the start
        # of a word - mon (monday)
        self.assertExpectedResult(self.cal.parse('money', start), (start, 0))
        # ensure short weekday names do not cause false positives at the end of
        # a word - th (thursday)
        self.assertExpectedResult(self.cal.parse('month', start), (start, 0))
        self.assertExpectedErrorFlag(
            self.cal.parse('30/030/01/071/07', start), (start, 0))
        # overflow due to Python's datetime
        self.assertExpectedResult(self.cal.parse('12345 y', start), (start, 0))
        self.assertExpectedResult(
            self.cal.parse('654321 w', start), (start, 0))
        self.assertExpectedResult(
            self.cal.parse('3700000 d', start), (start, 0))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test parsing of 'simple' offsets
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import calendar
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


def _truncateResult(result, trunc_seconds=True, trunc_hours=False):
    try:
        dt, flag = result
    except ValueError:
        # wtf?!
        return result
    if trunc_seconds:
        dt = dt[:5] + (0,) * 4
    if trunc_hours:
        dt = dt[:3] + (0,) * 6
    return dt, flag

_tr = _truncateResult


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testNow(self):
        s = datetime.datetime.now()

        start = s.timetuple()
        target = s.timetuple()

        self.assertExpectedResult(self.cal.parse('now', start), (target, 2))

    def testRightNow(self):
        s = datetime.datetime.now()

        start = s.timetuple()
        target = s.timetuple()

        self.assertExpectedResult(self.cal.parse('right now', start), (target, 2))

    def testOffsetFromDayOfWeek(self):
        self.cal.ptc.StartTimeFromSourceTime = True

        s = datetime.datetime(2016, 2, 16)  # a Tuesday
        t = datetime.datetime(2016, 2, 18)  # Thursday of the same week
        tPlusOffset = t + datetime.timedelta(hours=1)

        start = s.timetuple()
        target = t.timetuple()
        targetPlusOffset = tPlusOffset.timetuple()

        self.assertExpectedResult(
            self.cal.parse('Thursday', start), (target, 1))

        self.assertExpectedResult(
            self.cal.parse('one hour from Thursday', start), (targetPlusOffset, 3))

    def testOffsetBeforeDayOfWeek(self):
        self.cal.ptc.StartTimeFromSourceTime = True

        s = datetime.datetime(2016, 2, 16)  # a Tuesday
        t = datetime.datetime(2016, 2, 18)  # Thursday of the same week
        tPlusOffset = t + datetime.timedelta(hours=-1)

        start = s.timetuple()
        target = t.timetuple()
        targetPlusOffset = tPlusOffset.timetuple()

        self.assertExpectedResult(
            self.cal.parse('Thursday', start), (target, 1))

        self.assertExpectedResult(
            self.cal.parse('one hour before Thursday', start), (targetPlusOffset, 3))

    def testMinutesFromNow(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(minutes=5)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('5 minutes from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 min from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5m from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('in 5 minutes', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('in 5 min', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 minutes', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('5 min', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('5m', start), (target, 2))

        self.assertExpectedResult(
            self.cal.parse('five minutes from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five min from now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('in five minutes', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('in five min', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five minutes', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five min', start), (target, 2))

    def testMinutesBeforeNow(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(minutes=-5)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('5 minutes before now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 min before now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5m before now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 minutes ago', start), (target, 2))

        self.assertExpectedResult(
            self.cal.parse('five minutes before now', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five min before now', start), (target, 2))

    def testWeekFromNow(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(weeks=1)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('in 1 week', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 week from now', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('in one week', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('one week from now', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('in a week', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('a week from now', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('in 7 days', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('7 days from now', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('in seven days', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('seven days from now', start), (target, 3))
        self.assertEqual(_tr(self.cal.parse('next week', start),
                             trunc_hours=True),
                         _tr((target, 1), trunc_hours=True))

    def testNextWeekDay(self):
        start = datetime.datetime.now()
        target = start + datetime.timedelta(days=4 + 7 - start.weekday())
        start = start.timetuple()
        target = target.timetuple()

        self.assertExpectedResult(self.cal.parse('next friday', start),
                                  (target, 1), dateOnly=True)
        self.assertExpectedResult(self.cal.parse('next friday?', start),
                                  (target, 1), dateOnly=True)
        self.cal.ptc.StartTimeFromSourceTime = True
        self.assertExpectedResult(self.cal.parse('next friday', start),
                                  (target, 1))

    def testNextWeekDayWithTime(self):
        start = datetime.datetime.now()
        target = start + datetime.timedelta(days=4 + 7 - start.weekday())
        target = target.replace(hour=13, minute=0, second=0)
        target = target.timetuple()

        self.assertExpectedResult(self.cal.parse('next friday at 1pm', start),
                                  (target, 3))
        self.assertExpectedResult(self.cal.parse('1pm next friday', start),
                                  (target, 3))

        target = start + datetime.timedelta(days=4 - start.weekday())
        target = target.replace(hour=13, minute=0, second=0)
        target = target.timetuple()
        self.assertExpectedResult(self.cal.parse('1pm this friday', start),
                                  (target, 3))

    def testWeekBeforeNow(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(weeks=-1)

        start = s.timetuple()
        target = t.timetuple()

        self.assertEqual(_tr(self.cal.parse('1 week before now', start)),
                         _tr((target, 3)))
        self.assertEqual(_tr(self.cal.parse('one week before now', start)),
                         _tr((target, 3)))
        self.assertEqual(_tr(self.cal.parse('a week before now', start)),
                         _tr((target, 3)))
        self.assertEqual(_tr(self.cal.parse('7 days before now', start)),
                         _tr((target, 3)))
        self.assertEqual(_tr(self.cal.parse('seven days before now', start)),
                         _tr((target, 3)))
        self.assertEqual(_tr(self.cal.parse('1 week ago', start)),
                         _tr((target, 1)))
        self.assertEqual(_tr(self.cal.parse('a week ago', start)),
                         _tr((target, 1)))
        self.assertEqual(_tr(self.cal.parse('last week', start),
                             trunc_hours=True),
                         _tr((target, 1), trunc_hours=True))

    def testNextMonth(self):
        s = (datetime.datetime(self.yr, self.mth, self.dy,
                               self.hr, self.mn, self.sec) +
             datetime.timedelta(days=1))
        t = self.cal.inc(s, year=1)

        start = s.timetuple()
        target = t.timetuple()

        phrase = 'next %s %s' % (calendar.month_name[t.month], t.day)

        self.assertEqual(_tr(self.cal.parse(phrase, start)),
                         _tr((target, 1)))

    def testSpecials(self):
        s = datetime.datetime.now()
        t = datetime.datetime(
            self.yr, self.mth, self.dy, 9, 0, 0) + datetime.timedelta(days=1)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('tomorrow', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('next day', start), (target, 1))

        t = datetime.datetime(
            self.yr, self.mth, self.dy, 9, 0, 0) + datetime.timedelta(days=-1)
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('yesterday', start), (target, 1))

        t = datetime.datetime(self.yr, self.mth, self.dy, 9, 0, 0)
        target = t.timetuple()

        self.assertExpectedResult(self.cal.parse('today', start), (target, 1))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Examples of how to use parsedatetime with locale information provided.

Locale information can come from either PyICU (if available) or from
the more basic internal locale classes that are included in the
`Constants` class.
"""

__license__ = """
Copyright (c) 2004-2006 Mike Taylor
Copyright (c) 2006 Darshana Chhajed
All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import parsedatetime as pdt

# create an instance of Constants class so we can specify the locale

c = pdt.Constants("en")
p = pdt.Calendar(c)

# print out the values from Constants to show how the locale information
# is being used/stored internally

values = (c.uses24,                 # 24hr clock?
          c.usesMeridian,           # AM/PM used?
          c.usePyICU,               # was PyICU found/enabled?
          c.meridian,               # list of the am and pm values
          c.am,                     # list of the lowercase and stripped AM string
          c.pm,                     # list of the lowercase and stripped PM string
          c.dateFormats,            # dict of available date format strings
          c.timeFormats,            # dict of available time format strings
          c.timeSep,                # list of time separator, e.g. the ':' in '12:45'
          c.dateSep,                # list of date separator, e.g. the '/' in '11/23/2006'
          c.Months,                 # list of full month names
          c.shortMonths,            # list of the short month names
          c.Weekdays,               # list of the full week day names
          c.localeID                # the locale identifier
          )


print('\n'.join((str(value) for value in values)))


result = p.parse("March 24th")


# create an instance of Constants class and force it not to use PyICU
# and to use the internal Spanish locale class

c = pdt.Constants(localeID="es", usePyICU=False)
p = pdt.Calendar(c)

result = p.parse("Marzo 24")



# -*- coding: utf-8 -*-
"""
Test pdtContext
"""

import sys
import time
import parsedatetime as pdt
from parsedatetime.context import pdtContext

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    def setUp(self):
        self.cal = pdt.Calendar(version=pdt.VERSION_CONTEXT_STYLE)
        (self.yr, self.mth, self.dy, self.hr, self.mn,
         self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testContext(self):
        self.assertEqual(self.cal.parse('5 min from now')[1],
                         pdtContext(pdtContext.ACU_MIN | pdtContext.ACU_NOW))
        self.assertEqual(self.cal.parse('5 min from now',
                                        version=pdt.VERSION_FLAG_STYLE)[1], 2)
        self.assertEqual(self.cal.parse('7/11/2015')[1],
                         pdtContext(pdtContext.ACU_YEAR |
                                    pdtContext.ACU_MONTH | pdtContext.ACU_DAY))
        self.assertEqual(self.cal.parse('7/11/2015',
                                        version=pdt.VERSION_FLAG_STYLE)[1], 1)
        self.assertEqual(self.cal.parse('14/32/2015')[1],
                         pdtContext(0))
        self.assertEqual(self.cal.parse('25:23')[1],
                         pdtContext())

    def testSources(self):
        self.assertEqual(self.cal.parse('afternoon 5pm')[1],
                         pdtContext(pdtContext.ACU_HALFDAY |
                                    pdtContext.ACU_HOUR))

        self.assertEqual(self.cal.parse('morning')[1],
                         pdtContext(pdtContext.ACU_HALFDAY))

        self.assertEqual(self.cal.parse('night', version=1)[1], 2)

    def testThreadRun(self):
        from threading import Thread
        t = Thread(target=lambda: self.cal.evalRanges('4p-6p'))
        # should not throw out AttributeError
        t.start()


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test parsing of simple date and times
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import string
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testDays(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(days=1)

        start = s.timetuple()
        target = t.timetuple()

        d = self.wd + 1

        if d > 6:
            d = 0

        day = self.cal.ptc.Weekdays[d]

        self.assertExpectedResult(self.cal.parse(day, start), (target, 1))

        t = s + datetime.timedelta(days=6)

        target = t.timetuple()

        d = self.wd - 1

        if d < 0:
            d = 6

        day = self.cal.ptc.Weekdays[d]

        self.assertExpectedResult(self.cal.parse(day, start), (target, 1))

    def testTimes(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            self.yr, self.mth, self.dy, 23, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('11:00:00 PM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11:00 PM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11 PM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11PM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('2300', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('23:00', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11p', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11:00:00 P.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11:00 P.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11 P.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11P.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11p.m.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11 p.m.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('"11 p.m."', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 11, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('11:00:00 AM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11:00 AM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11 AM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11AM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1100', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11:00', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11a', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11am', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11:00:00 A.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11:00 A.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11 A.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11A.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11a.m.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('11 a.m.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('(11 a.m.)', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 7, 30, 0).timetuple()

        self.assertExpectedResult(self.cal.parse('730', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('0730', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('0730am', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 17, 30, 0).timetuple()

        self.assertExpectedResult(self.cal.parse('1730', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('173000', start), (target, 2))

        # Should not parse as a time due to prefix
        self.assertExpectedResult(self.cal.parse('$300', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('300ml', start), (start, 0))

        # Should not parse as a time due to false meridian
        self.assertExpectedResult(self.cal.parse('3 axmx', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('3 pxmx', start), (start, 0))

    def testDates(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            2006, 8, 25, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('08/25/2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('08.25.2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('2006/08/25', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('2006/8/25', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('2006-08-25', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('8/25/06', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('August 25, 2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug 25, 2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug. 25, 2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('August 25 2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug 25 2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug. 25 2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('25 August 2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('25 Aug 2006', start), (target, 1))

        if self.mth > 8 or (self.mth == 8 and self.dy > 25):
            target = datetime.datetime(
                self.yr + 1, 8, 25, self.hr, self.mn, self.sec).timetuple()
        else:
            target = datetime.datetime(
                self.yr, 8, 25, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('8/25', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('8.25', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('08/25', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('August 25', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug 25', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug. 25', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('"8.25"', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('(8.25)', start), (target, 1))

        # Should not parse as dates
        self.assertExpectedResult(self.cal.parse('$1.23', start), (start, 0))
        self.assertExpectedResult(self.cal.parse('$12.34', start), (start, 0))

        # added test to ensure 4-digit year is recognized in the absence of day
        target = datetime.datetime(
            2013, 8, 1, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('Aug. 2013', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug  2013', start), (target, 1))

    def testLeapDays(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            2000, 2, 29, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('02/29/2000', start), (target, 1))

        target = datetime.datetime(
            2004, 2, 29, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('02/29/2004', start), (target, 1))

        target = datetime.datetime(
            2008, 2, 29, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('02/29/2008', start), (target, 1))

        target = datetime.datetime(
            2012, 2, 29, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('02/29/2012', start), (target, 1))

        dNormal = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
        dLeap = (31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)

        for i in range(1, 12):
            self.assertTrue(self.cal.ptc.daysInMonth(i, 1999), dNormal[i - 1])
            self.assertTrue(self.cal.ptc.daysInMonth(i, 2000), dLeap[i - 1])
            self.assertTrue(self.cal.ptc.daysInMonth(i, 2001), dNormal[i - 1])
            self.assertTrue(self.cal.ptc.daysInMonth(i, 2002), dNormal[i - 1])
            self.assertTrue(self.cal.ptc.daysInMonth(i, 2003), dNormal[i - 1])
            self.assertTrue(self.cal.ptc.daysInMonth(i, 2004), dLeap[i - 1])
            self.assertTrue(self.cal.ptc.daysInMonth(i, 2005), dNormal[i - 1])

    def testDaySuffixes(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            2008, 8, 22, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('August 22nd, 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug 22nd, 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug. 22nd, 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('August 22nd 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug 22nd 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug. 22nd 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('22nd August 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('22nd Aug 2008', start), (target, 1))

        target = datetime.datetime(
            1949, 12, 31, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('December 31st, 1949', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Dec 31st, 1949', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('December 31st 1949', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Dec 31st 1949', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('31st December 1949', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('31st Dec 1949', start), (target, 1))

        target = datetime.datetime(
            2008, 8, 23, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('August 23rd, 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug 23rd, 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug. 23rd, 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('August 23rd 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug 23rd 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug. 23rd 2008', start), (target, 1))

        target = datetime.datetime(
            2008, 8, 25, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('August 25th, 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug 25th, 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug. 25th, 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('August 25th 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug 25th 2008', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('Aug. 25th 2008', start), (target, 1))

    def testSpecialTimes(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            self.yr, self.mth, self.dy, 6, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('morning', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 8, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('breakfast', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 12, 0, 0).timetuple()

        self.assertExpectedResult(self.cal.parse('lunch', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 13, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('afternoon', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 18, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('evening', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 19, 0, 0).timetuple()

        self.assertExpectedResult(self.cal.parse('dinner', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 21, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('night', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('tonight', start), (target, 2))

    def testMidnight(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            self.yr, self.mth, self.dy, 0, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('midnight', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12:00:00 AM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12:00 AM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12 AM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12AM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12am', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12a', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('0000', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('00:00', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12:00:00 A.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12:00 A.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12 A.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12A.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12a.m.', start), (target, 2))

    def testNoon(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            self.yr, self.mth, self.dy, 12, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('noon', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12:00:00 PM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12:00 PM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12 PM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12PM', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12p', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1200', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12:00', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12:00:00 P.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12:00 P.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12 P.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12P.M.', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('12p.m.', start), (target, 2))

    def testDaysOfWeek(self):
        start = datetime.datetime(
            2014, 10, 25, self.hr, self.mn, self.sec).timetuple()

        target = datetime.datetime(
            2014, 10, 26, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('sunday', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('sun', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 27, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('Monday', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('mon', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 28, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('tuesday', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('tues', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 29, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('wednesday', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('wed', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 30, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('thursday', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('thu', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 31, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('friday', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('fri', start), (target, 1))

        target = datetime.datetime(
            2014, 11, 1, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('saturday', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('sat', start), (target, 1))

    def testWordBoundaries(self):
        # Ensure that keywords appearing at the start of a word are not parsed
        # as if they were standalone keywords. For example, "10 dogs" should
        # not be interpreted the same as "10 d"
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime.now().timetuple()

        keywords = []
        loc = self.cal.ptc.locale

        def flattenWeekdays(wds):
            return sum([wd.split('|') for wd in wds], [])

        # Test all known keywords for the locale
        keywords.extend(loc.meridian)
        keywords.extend(flattenWeekdays(loc.Weekdays))
        keywords.extend(flattenWeekdays(loc.shortWeekdays))
        keywords.extend(loc.Months)
        keywords.extend(loc.shortMonths)
        keywords.extend(loc.numbers.keys())
        keywords.extend(loc.Modifiers.keys())
        keywords.extend(loc.dayOffsets.keys())
        keywords.extend(loc.re_sources.keys())
        keywords.extend(loc.small.keys())
        keywords.extend(loc.magnitude.keys())

        for units in loc.units.values():
            keywords.extend(units)

        # Finally, test all lowercase letters to be particularly thorough - it
        # would be very difficult to track down bugs due to single letters.
        keywords.extend(list(string.ascii_lowercase))

        for keyword in keywords:
            phrase = '1 %sfoo' % keyword
            self.assertExpectedResult(
                self.cal.parse(phrase, start), (target, 0),
                'Result does not match target value: %s' % repr(phrase))

    def testYearParseStyle(self):
        config = pdt.Constants()
        config.YearParseStyle = 0
        calendar = pdt.Calendar(config)
        start = datetime.datetime(self.yr, self.mth, self.dy,
                                  self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(self.yr, 7, 28,
                                   self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(calendar.parse('7/28', start), (target, 1))

    # def testMonths(self):

    #     start = datetime.datetime(
    #         self.yr, self.mth, self.dy,
    #         self.hr, self.mn, self.sec).timetuple()

    #     target = datetime.datetime(
    #         self.yr, self.mth, self.dy, 12, 0, 0).timetuple()

    #     self.assertExpectedResult(self.cal.parse('jun', start), (target, 2))
    #     self.assertExpectedResult(
    #         self.cal.parse('12:00:00 PM', start), (target, 2))
    #     self.assertExpectedResult(
    #         self.cal.parse('12:00 PM', start), (target, 2))
    #     self.assertExpectedResult(self.cal.parse('12 PM', start),
    #                               (target, 2))
    #     self.assertExpectedResult(self.cal.parse('12PM', start), (target, 2))
    #     self.assertExpectedResult(self.cal.parse('12pm', start), (target, 2))
    #     self.assertExpectedResult(self.cal.parse('12p', start), (target, 2))
    #     self.assertExpectedResult(self.cal.parse('1200', start), (target, 2))
    #     self.assertExpectedResult(self.cal.parse('12:00', start),
    #                               (target, 2))

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test parsing of 'simple' offsets
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testOffsetAfterNoon(self):
        s = datetime.datetime(self.yr, self.mth, self.dy, 10, 0, 0)
        t = datetime.datetime(
            self.yr, self.mth, self.dy, 12, 0, 0) + datetime.timedelta(hours=5)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('5 hours after 12pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five hours after 12pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours after 12 pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours after 12:00pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours after 12:00 pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours after noon', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours from noon', start), (target, 2))

    def testOffsetBeforeNoon(self):
        s = datetime.datetime.now()
        t = (datetime.datetime(self.yr, self.mth, self.dy, 12, 0, 0) +
             datetime.timedelta(hours=-5))

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('5 hours before noon', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours before 12pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('five hours before 12pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours before 12 pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours before 12:00pm', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('5 hours before 12:00 pm', start), (target, 2))

    def testOffsetBeforeModifiedNoon(self):
        # A contrived test of two modifiers applied to noon - offset by
        # -5 from the following day (-5 + 24)
        s = datetime.datetime.now()
        t = (datetime.datetime(self.yr, self.mth, self.dy, 12, 0, 0) +
             datetime.timedelta(hours=-5 + 24))

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('5 hours before next noon', start), (target, 2))

if __name__ == "__main__":
    unittest.main()



"""
Basic examples of how to use parsedatetime
"""

__license__ = """
Copyright (c) 2004-2006 Mike Taylor
Copyright (c) 2006 Darshana Chhajed
All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import parsedatetime as pdt

# create an instance of Constants class so we can override some of the defaults

c = pdt.Constants()

c.BirthdayEpoch = 80    # BirthdayEpoch controls how parsedatetime
                        # handles two digit years.  If the parsed
                        # value is less than this value then the year
                        # is set to 2000 + the value, otherwise
                        # it's set to 1900 + the value

# create an instance of the Calendar class and pass in our Constants
# object instead of letting it create a default

p = pdt.Calendar(c)

# parse "tomorrow" and return the result

result = p.parse("tomorrow")

# parseDate() is a helper function that bypasses all of the
# natural language stuff and just tries to parse basic dates
# but using the locale information

result = p.parseDate("4/4/80")

# parseDateText() is a helper function that tries to parse
# long-form dates using the locale information

result = p.parseDateText("March 5th, 1980")




# -*- coding: utf-8 -*-
"""
Test parsing of strings that are phrases
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testPhrases(self):
        #
        # NOTE - this test will fail under certain conditions
        #        It is building an absolute date for comparison and then
        #        testing the parsing of relative phrases and as such will fail
        #        if run near the midnight transition.
        #        Thanks to Chris Petrilli for asking about it and prompting me
        #        to create this note!
        #
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            self.yr, self.mth, self.dy, 16, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('flight from SFO at 4pm', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 17, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('eod', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('meeting eod', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('eod meeting', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 17, 0, 0) + datetime.timedelta(days=1)
        target = target.timetuple()

        self.assertExpectedResult(
            self.cal.parse('tomorrow eod', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('eod tomorrow', start), (target, 3))

    def testPhraseWithDays_DOWStyle_1_False(self):
        s = datetime.datetime.now()

        # find out what day we are currently on
        # and determine what the next day of week is
        t = s + datetime.timedelta(days=1)
        start = s.timetuple()

        (yr, mth, dy, _, _, _, wd, yd, isdst) = t.timetuple()

        target = (yr, mth, dy, 17, 0, 0, wd, yd, isdst)

        d = self.wd + 1
        if d > 6:
            d = 0

        day = self.cal.ptc.Weekdays[d]

        self.assertExpectedResult(
            self.cal.parse('eod %s' % day, start), (target, 3))

        # find out what day we are currently on
        # and determine what the previous day of week is
        t = s + datetime.timedelta(days=6)

        (yr, mth, dy, hr, mn, sec, wd, yd, isdst) = t.timetuple()

        target = (yr, mth, dy, 17, 0, 0, wd, yd, isdst)

        d = self.wd - 1
        if d < 0:
            d = 6

        day = self.cal.ptc.Weekdays[d]

        self.assertExpectedResult(
            self.cal.parse('eod %s' % day, start), (target, 3))

    def testEndOfPhrases(self):
        s = datetime.datetime.now()

        # find out what month we are currently on
        # set the day to 1 and then go back a day
        # to get the end of the current month
        (yr, mth, _, hr, mn, sec, _, _, _) = s.timetuple()

        mth += 1
        if mth > 12:
            mth = 1
            yr += 1

        t = datetime.datetime(
            yr, mth, 1, 9, 0, 0) + datetime.timedelta(days=-1)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('eom', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('meeting eom', start), (target, 1))

        s = datetime.datetime.now()

        (yr, mth, dy, hr, mn, sec, wd, yd, isdst) = s.timetuple()

        t = datetime.datetime(yr, 12, 31, 9, 0, 0)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('eoy', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('meeting eoy', start), (target, 1))

    def testLastPhrases(self):
        for day in (11, 12, 13, 14, 15, 16, 17):
            start = datetime.datetime(2012, 11, day, 9, 0, 0)

            (yr, mth, dy, _, _, _, wd, yd, isdst) = start.timetuple()

            n = 4 - wd
            if n >= 0:
                n -= 7

            target = start + datetime.timedelta(days=n)

            self.assertExpectedResult(
                self.cal.parse('last friday', start.timetuple()),
                (target.timetuple(), 1), dateOnly=True)



# -*- coding: utf-8 -*-
"""
Test parsing of units
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testMinutes(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(minutes=1)
        h = s - datetime.timedelta(minutes=1)

        start = s.timetuple()
        target = t.timetuple()
        history = h.timetuple()

        self.assertExpectedResult(
            self.cal.parse('1 minutes', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1 minute', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1 min', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1min', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1 m', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1m', start), (target, 2))

        self.assertExpectedResult(
            self.cal.parse('1 minutes ago', start), (history, 2))
        self.assertExpectedResult(
            self.cal.parse('1 minute ago', start), (history, 2))

    def testHours(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(hours=1)
        h = s - datetime.timedelta(hours=1)

        start = s.timetuple()
        target = t.timetuple()
        history = h.timetuple()

        self.assertExpectedResult(
            self.cal.parse('1 hour', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1 hours', start), (target, 2))
        self.assertExpectedResult(
            self.cal.parse('1 hr', start), (target, 2))

        self.assertExpectedResult(
            self.cal.parse('1 hour ago', start), (history, 2))
        self.assertExpectedResult(
            self.cal.parse('1 hours ago', start), (history, 2))

    def testDays(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(days=1)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(self.cal.parse('1 day', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('1 days', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('1days', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('1 dy', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('1 d', start), (target, 1))

    def testNegativeDays(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(days=-1)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('-1 day', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('-1 days', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('-1days', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('-1 dy', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('-1 d', start), (target, 1))

        self.assertExpectedResult(
            self.cal.parse('- 1 day', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('- 1 days', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('- 1days', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('- 1 dy', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('- 1 d', start), (target, 1))

        self.assertExpectedResult(
            self.cal.parse('1 day ago', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 days ago', start), (target, 1))

    def testWeeks(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(weeks=1)
        h = s - datetime.timedelta(weeks=1)

        start = s.timetuple()
        target = t.timetuple()
        history = h.timetuple()

        self.assertExpectedResult(
            self.cal.parse('1 week', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1week', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 weeks', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 wk', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 w', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1w', start), (target, 1))

        self.assertExpectedResult(
            self.cal.parse('1 week ago', start), (history, 1))
        self.assertExpectedResult(
            self.cal.parse('1 weeks ago', start), (history, 1))

    def testMonths(self):
        s = datetime.datetime.now()
        t = self.cal.inc(s, month=1)
        h = self.cal.inc(s, month=-1)

        start = s.timetuple()
        target = t.timetuple()
        history = h.timetuple()

        self.assertExpectedResult(
            self.cal.parse('1 month', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 months', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1month', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 month ago', start), (history, 1))
        self.assertExpectedResult(
            self.cal.parse('1 months ago', start), (history, 1))

    def testYears(self):
        s = datetime.datetime.now()
        t = self.cal.inc(s, year=1)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('1 year', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 years', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 yr', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1 y', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('1y', start), (target, 1))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import sys
import time
import datetime
import parsedatetime as pdt
from parsedatetime.pdt_locales import get_icu
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


pdtLocale_en = get_icu('en_US')
pdtLocale_en.Weekdays = [
    'monday', 'tuesday', 'wednesday',
    'thursday', 'friday', 'saturday', 'sunday']

pdtLocale_en.shortWeekdays = [
    'mon|mond', 'tue|tues', 'wed|wedn',
    'thu|thur|thurs', 'fri|frid', 'sat|sa', 'sun|su']

pdtLocale_en.Months = [
    'january', 'february', 'march', 'april', 'may', 'june',
    'july', 'august', 'september', 'october', 'november', 'december']

pdtLocale_en.shortMonths = [
    'jan|janu', 'feb|febr', 'mar|marc', 'apr|apri', 'may', 'jun|june',
    'jul', 'aug|augu', 'sep|sept', 'oct|octo', 'nov|novem', 'dec|decem']


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        pdt.pdtLocales['en_us'] = pdtLocale_en  # override for the test
        self.ptc = pdt.Constants('en_us', usePyICU=False)
        self.cal = pdt.Calendar(self.ptc)
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testDaysOfWeek(self):
        start = datetime.datetime(
            2014, 10, 25, self.hr, self.mn, self.sec).timetuple()

        target = datetime.datetime(
            2014, 10, 26, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(self.cal.parse('sunday', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('sun', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('su', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 27, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(self.cal.parse('Monday', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('mon', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('mond', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 28, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('tuesday', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('tues', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('tue', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 29, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('wednesday', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('wedn', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('wed', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 30, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('thursday', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('thu', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('thur', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('thurs', start), (target, 1))

        target = datetime.datetime(
            2014, 10, 31, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(self.cal.parse('friday', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('fri', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('frid', start), (target, 1))

        target = datetime.datetime(
            2014, 11, 1, self.hr, self.mn, self.sec).timetuple()
        self.assertExpectedResult(
            self.cal.parse('saturday', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('sat', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('sa', start), (target, 1))

    def testMonths(self):
        start = datetime.datetime(
            2014, 1, 1, self.hr, self.mn, self.sec).timetuple()
        for dates, expected_date in [
            ('jan|janu|january', datetime.datetime(
                2014, 1, 1, self.hr, self.mn, self.sec).timetuple()),
            ('feb|febr|february', datetime.datetime(
                2014, 2, 1, self.hr, self.mn, self.sec).timetuple()),
            ('mar|marc|march', datetime.datetime(
                2014, 3, 1, self.hr, self.mn, self.sec).timetuple()),
            ('apr|apri|april', datetime.datetime(
                2014, 4, 1, self.hr, self.mn, self.sec).timetuple()),
            ('may|may', datetime.datetime(
                2014, 5, 1, self.hr, self.mn, self.sec).timetuple()),
            ('jun|june', datetime.datetime(
                2014, 6, 1, self.hr, self.mn, self.sec).timetuple()),
            ('jul|july', datetime.datetime(
                2014, 7, 1, self.hr, self.mn, self.sec).timetuple()),
            ('aug|augu|august', datetime.datetime(
                2014, 8, 1, self.hr, self.mn, self.sec).timetuple()),
            ('sep|sept|september', datetime.datetime(
                2014, 9, 1, self.hr, self.mn, self.sec).timetuple()),
            ('oct|octo|october', datetime.datetime(
                2014, 10, 1, self.hr, self.mn, self.sec).timetuple()),
            ('nov|novem|november', datetime.datetime(
                2014, 11, 1, self.hr, self.mn, self.sec).timetuple()),
            ('dec|decem|december', datetime.datetime(
                2014, 12, 1, self.hr, self.mn, self.sec).timetuple())
        ]:
            for dateText in dates.split("|"):
                # print dateText
                self.assertExpectedResult(
                    self.cal.parse(dateText, start), (expected_date, 1))



# -*- coding: utf-8 -*-
"""
Test parsing of strings with multiple chunks
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.cal = pdt.Calendar()
        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testSimpleMultipleItems(self):
        s = datetime.datetime.now()
        t = self.cal.inc(s, year=3) + datetime.timedelta(days=5, weeks=2)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('3 years 2 weeks 5 days', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('3years 2weeks 5days', start), (target, 1))

    def testMultipleItemsSingleCharUnits(self):
        s = datetime.datetime.now()
        t = self.cal.inc(s, year=3) + datetime.timedelta(days=5, weeks=2)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('3 y 2 w 5 d', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('3y 2w 5d', start), (target, 1))

        t = self.cal.inc(s, year=3) + datetime.timedelta(hours=5, minutes=50)
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('3y 5h 50m', start), (target, 3))

    def testMultipleItemsWithPunctuation(self):
        s = datetime.datetime.now()
        t = self.cal.inc(s, year=3) + datetime.timedelta(days=5, weeks=2)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('3 years, 2 weeks, 5 days', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('3 years, 2 weeks and 5 days', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('3y, 2w, 5d ', start), (target, 1))

    def testUnixATStyle(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(days=3)

        t = t.replace(hour=16, minute=0, second=0)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('4pm + 3 days', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('4pm +3 days', start), (target, 3))

    def testUnixATStyleNegative(self):
        s = datetime.datetime.now()
        t = s + datetime.timedelta(days=-3)

        t = t.replace(hour=16, minute=0, second=0)

        start = s.timetuple()
        target = t.timetuple()

        self.assertExpectedResult(
            self.cal.parse('4pm - 3 days', start), (target, 3))
        self.assertExpectedResult(
            self.cal.parse('4pm -3 days', start), (target, 3))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test parsing of simple date and times using the Russian locale
"""
from __future__ import unicode_literals

import sys
import time
import datetime
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        locale = 'ru_RU'
        self.ptc = pdt.Constants(locale, usePyICU=False)
        self.cal = pdt.Calendar(self.ptc)

        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

        if self.ptc.localeID != locale:
            raise unittest.SkipTest(
                'Locale not set to %s - check if PyICU is installed' % locale)

    def testTimes(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            self.yr, self.mth, self.dy, 23, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('23:00:00', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('23:00', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('2300', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 11, 0, 0).timetuple()

        self.assertExpectedResult(
            self.cal.parse('11:00:00', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('11:00', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('1100', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 7, 30, 0).timetuple()

        self.assertExpectedResult(self.cal.parse('730', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('0730', start), (target, 2))

        target = datetime.datetime(
            self.yr, self.mth, self.dy, 17, 30, 0).timetuple()

        self.assertExpectedResult(self.cal.parse('1730', start), (target, 2))
        self.assertExpectedResult(self.cal.parse('173000', start), (target, 2))

    def testDates(self):
        start = datetime.datetime(
            self.yr, self.mth, self.dy, self.hr, self.mn, self.sec).timetuple()
        target = datetime.datetime(
            2006, 8, 25, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(
            self.cal.parse('25.08.2006', start), (target, 1))
        self.assertExpectedResult(
            self.cal.parse('25.8.06', start), (target, 1))

        if self.mth > 8 or (self.mth == 8 and self.dy > 25):
            target = datetime.datetime(
                self.yr + 1, 8, 25, self.hr, self.mn, self.sec).timetuple()
        else:
            target = datetime.datetime(
                self.yr, 8, 25, self.hr, self.mn, self.sec).timetuple()

        self.assertExpectedResult(self.cal.parse('25.8', start), (target, 1))
        self.assertExpectedResult(self.cal.parse('25.08', start), (target, 1))

    def testDatesLang(self):
        if sys.version_info >= (3, 0):
            target = datetime.datetime(2006, 8, 25, 23, 5).timetuple()
            self.assertExpectedResult(
                self.cal.parse('25 августа 2006 23:05'), (target, 3))
            target = datetime.datetime(
                2006, 8, 25, self.hr, self.mn, self.sec).timetuple()
            self.assertExpectedResult(
                self.cal.parse('25 августа 2006'), (target, 1))
        self.assertEqual(self.cal.parse('23:05')[0][3], 23)
        self.assertEqual(self.cal.parse('23:05')[0][4], 5)

    def testConjugate(self):
        if sys.version_info >= (3, 0):
            target = datetime.datetime(2006, 9, 25, 23, 5).timetuple()
            self.assertExpectedResult(
                self.cal.parse('25 сентября 2006 23:05'), (target, 3))
            # self.assertExpectedResult(
            #     self.cal.parse('25 сентябрь 2006 23:05'), (target, 3))

    # does not work with travis
    # datetime.now() return non correct data
    # def testdayOffsets(self):
    #     def get_datetime(tuple_time):
    #         return datetime.datetime(*tuple_time[:6]).date()
    #
    #     now = datetime.datetime.today().date()
    #
    #     self.assertEqual(
    #         get_datetime(self.cal.parse("вчера")[0]),
    #         now - datetime.timedelta(days=1)
    #     )
    #     self.assertEqual(
    #         get_datetime(self.cal.parse("завтра")[0]),
    #         now + datetime.timedelta(days=1)
    #     )
    #
    #     self.assertEqual(
    #         get_datetime(self.cal.parse("позавчера")[0]),
    #         now - datetime.timedelta(days=2)
    #     )
    #
    #     self.assertEqual(
    #         get_datetime(self.cal.parse("послезавтра")[0]),
    #         now + datetime.timedelta(days=2)
    #     )

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Test parsing of simple date and times using the French locale

Note: requires PyICU
"""
from __future__ import unicode_literals
import sys
import time
import datetime
import pytest
import parsedatetime as pdt
from . import utils

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):

    @utils.assertEqualWithComparator
    def assertExpectedResult(self, result, check, **kwargs):
        return utils.compareResultByTimeTuplesAndFlags(result, check, **kwargs)

    def setUp(self):
        self.ptc = pdt.Constants('fr_FR')
        self.cal = pdt.Calendar(self.ptc)

        (self.yr, self.mth, self.dy, self.hr,
         self.mn, self.sec, self.wd, self.yd, self.isdst) = time.localtime()

    def testTimes(self):
        if self.ptc.localeID == 'fr_FR':
            start = datetime.datetime(
                self.yr, self.mth, self.dy,
                self.hr, self.mn, self.sec).timetuple()
            target = datetime.datetime(
                self.yr, self.mth, self.dy, 23, 0, 0).timetuple()

            self.assertExpectedResult(
                self.cal.parse('2300', start), (target, 2))
            self.assertExpectedResult(
                self.cal.parse('23:00', start), (target, 2))

            target = datetime.datetime(
                self.yr, self.mth, self.dy, 11, 0, 0).timetuple()

            self.assertExpectedResult(
                self.cal.parse('1100', start), (target, 2))
            self.assertExpectedResult(
                self.cal.parse('11:00', start), (target, 2))

            target = datetime.datetime(
                self.yr, self.mth, self.dy, 7, 30, 0).timetuple()

            self.assertExpectedResult(
                self.cal.parse('730', start), (target, 2))
            self.assertExpectedResult(
                self.cal.parse('0730', start), (target, 2))

            target = datetime.datetime(
                self.yr, self.mth, self.dy, 17, 30, 0).timetuple()

            self.assertExpectedResult(
                self.cal.parse('1730', start), (target, 2))
            self.assertExpectedResult(
                self.cal.parse('173000', start), (target, 2))

    def testDates(self):
        if self.ptc.localeID == 'fr_FR':
            start = datetime.datetime(
                self.yr, self.mth, self.dy,
                self.hr, self.mn, self.sec).timetuple()
            target = datetime.datetime(
                2006, 8, 25, self.hr, self.mn, self.sec).timetuple()

            self.assertExpectedResult(
                self.cal.parse('25/08/2006', start), (target, 1))
            self.assertExpectedResult(
                self.cal.parse('25/8/06', start), (target, 1))
            self.assertExpectedResult(
                self.cal.parse('août 25, 2006', start), (target, 1))
            self.assertExpectedResult(
                self.cal.parse('août 25 2006', start), (target, 1))

            if self.mth > 8 or (self.mth == 8 and self.dy > 25):
                target = datetime.datetime(
                    self.yr + 1, 8, 25, self.hr, self.mn, self.sec).timetuple()
            else:
                target = datetime.datetime(
                    self.yr, 8, 25, self.hr, self.mn, self.sec).timetuple()

            self.assertExpectedResult(
                self.cal.parse('25/8', start), (target, 1))
            self.assertExpectedResult(
                self.cal.parse('25/08', start), (target, 1))

    def testWeekDays(self):
        if self.ptc.localeID == 'fr_FR':
            start = datetime.datetime(
                self.yr, self.mth, self.dy,
                self.hr, self.mn, self.sec).timetuple()

            o1 = self.ptc.CurrentDOWParseStyle
            o2 = self.ptc.DOWParseStyle

            # set it up so the current dow returns current day
            self.ptc.CurrentDOWParseStyle = True
            self.ptc.DOWParseStyle = 1

            for i in range(0, 7):
                dow = self.ptc.shortWeekdays[i]

                result = self.cal.parse(dow, start)

                yr, mth, dy, hr, mn, sec, wd, yd, isdst = result[0]

                self.assertEqual(wd, i)

            self.ptc.CurrentDOWParseStyle = o1
            self.ptc.DOWParseStyle = o2


class TestDayOffsets(test):
    # test how Aujourd'hui/Demain/Hier are parsed

    def setUp(self):
        super(TestDayOffsets, self).setUp()
        self.ptc = pdt.Constants('fr_FR', usePyICU=False)
        self.cal = pdt.Calendar(self.ptc)

    def test_dayoffsets(self):
        start = datetime.datetime(self.yr, self.mth, self.dy, 9)
        for date_string, expected_day_offset in [
                ("Aujourd'hui", 0),
                ("aujourd'hui", 0),
                ("Demain", 1),
                ("demain", 1),
                ("Hier", -1),
                ("hier", -1),
                ("au jour de hui", None)]:
            got_dt, rc = self.cal.parseDT(date_string, start)
            if expected_day_offset is not None:
                self.assertEqual(rc, 1)
                target = (start + datetime.timedelta(days=expected_day_offset))
                self.assertEqual(got_dt, target)
            else:
                self.assertEqual(rc, 0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import codecs

from setuptools import setup, find_packages

cwd = os.path.abspath(os.path.dirname(__file__))

def read(filename):
    with codecs.open(os.path.join(cwd, filename), 'rb', 'utf-8') as h:
        return h.read()

metadata = read(os.path.join(cwd, 'parsedatetime', '__init__.py'))

def extract_metaitem(meta):
    # swiped from https://hynek.me 's attr package
    meta_match = re.search(r"""^__{meta}__\s+=\s+['\"]([^'\"]*)['\"]""".format(meta=meta),
                           metadata, re.MULTILINE)
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError('Unable to find __{meta}__ string.'.format(meta=meta))

setup(
    name='parsedatetime',
    version=extract_metaitem('version'),
    author=extract_metaitem('author'),
    author_email=extract_metaitem('email'),
    url=extract_metaitem('url'),
    download_url=extract_metaitem('download_url'),
    description=extract_metaitem('description'),
    license=extract_metaitem('license'),
    packages=find_packages(exclude=['tests', 'docs']),
    platforms=['Any'],
    long_description=read('README.rst'),
    install_requires=['future'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite='tests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Text Processing',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ]
)



# -*- coding: utf-8 -*-
"""
Test time delta
"""

import sys
import time
import datetime
import parsedatetime as pdt

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


# added to support Python 2.6 which does not have total_seconds() method for timedelta
def total_seconds(timedelta):
    return (timedelta.microseconds + 0.0 +
            (timedelta.seconds + timedelta.days * 24 * 3600) * 10 ** 6) / 10 ** 6


class test(unittest.TestCase):

    def setUp(self):
        self.cal = pdt.Calendar(version=pdt.VERSION_CONTEXT_STYLE)
        self.source = (self.yr, self.mth, self.dy,
                       self.hr, self.mn, self.sec,
                       self.wd, self.yd, self.isdst) = time.localtime()

    def assertDelta(self, ts, years=None, months=None, **deltakw):
        ts = ts[0]
        source = self.source
        delta = datetime.timedelta(**deltakw)
        calc_delta = (datetime.datetime(*ts[:6]) -
                      datetime.datetime(*source[:6]))
        delta -= datetime.timedelta(microseconds=delta.microseconds)
        if not years and not months:
            self.assertEqual(delta, calc_delta)
            return
        if years:
            delta += datetime.timedelta(days=365 * years)
        if months:
            delta += datetime.timedelta(days=30 * months)
        diff = abs((total_seconds(calc_delta) -
                    total_seconds(delta)) /
                   total_seconds(delta))
        self.assertTrue(diff < 0.05, '%s is not less than 0.05' % diff)

    def testInteger(self):
        self.assertDelta(
            self.cal.parse('5 minutes ago', self.source), minutes=-5)
        self.assertDelta(
            self.cal.parse('34 hours ago', self.source), hours=-34)
        self.assertDelta(
            self.cal.parse('2 days ago', self.source), days=-2)

    def testFloat(self):
        self.assertDelta(
            self.cal.parse('58.4 minutes ago', self.source), minutes=-58.4)
        self.assertDelta(
            self.cal.parse('1855336.424 minutes ago', self.source),
            minutes=-1855336.424)
        self.assertDelta(
            self.cal.parse('8.3 hours ago', self.source), hours=-8.3)
        self.assertDelta(
            self.cal.parse('22.355 hours ago', self.source), hours=-22.355)
        self.assertDelta(
            self.cal.parse('7.2 days ago', self.source), days=-7.2)
        self.assertDelta(
            self.cal.parse('7.3 days ago', self.source), days=-7.3)
        self.assertDelta(
            self.cal.parse('17.7 days ago', self.source), days=-17.7)
        self.assertDelta(
            self.cal.parse('1.4 months ago', self.source), months=-1.4)
        self.assertDelta(
            self.cal.parse('4.8 months ago', self.source), months=-4.8)
        self.assertDelta(
            self.cal.parse('5.1 months ago', self.source), months=-5.1)
        self.assertDelta(
            self.cal.parse('5.11553 years ago', self.source), years=-5.11553)


if __name__ == "__main__":
    unittest.main()



"""
Tests the _convertUnitAsWords method.
"""

import sys
import parsedatetime as pdt

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest


class test(unittest.TestCase):
    def setUp(self):
        self.cal = pdt.Calendar()
        self.tests = (('one', 1),
                      ('zero', 0),
                      ('eleven', 11),
                      ('forty two', 42),
                      ('a hundred', 100),
                      ('four hundred and fifteen', 415),
                      ('twelve thousand twenty', 12020),
                      ('nine hundred and ninety nine', 999),
                      ('three quintillion four billion', 3000000004000000000),
                      ('forty three thousand, nine hundred and ninety nine', 43999),
                      ('one hundred thirty three billion four hundred thousand three hundred fourteen', 133000400314),
                      ('an octillion', 1000000000000000000000000000)
                      )

    def testConversions(self):
        for pair in self.tests:
            self.assertEqual(self.cal._convertUnitAsWords(pair[0]), pair[1])

if __name__ == "__main__":
    unittest.main()
